@@ -1,5 +1,5 @@
 # coding: utf-8
-"""A tornado based IPython notebook server."""
+"""A tornado based Jupyter notebook server."""
 
 # Copyright (c) Jupyter Development Team.
 # Distributed under the terms of the Modified BSD License.
@@ -13,18 +13,17 @@ import importlib
 import io
 import json
 import logging
+import mimetypes
 import os
 import random
 import re
 import select
 import signal
 import socket
-import ssl
 import sys
 import threading
 import webbrowser
 
-
 from jinja2 import Environment, FileSystemLoader
 
 # Install the pyzmq ioloop. This has to be done before anything else from
@@ -33,7 +32,7 @@ from zmq.eventloop import ioloop
 ioloop.install()
 
 # check for tornado 3.1.0
-msg = "The IPython Notebook requires tornado >= 4.0"
+msg = "The Jupyter Notebook requires tornado >= 4.0"
 try:
     import tornado
 except ImportError:
@@ -77,23 +76,21 @@ from jupyter_client.session import Session
 from nbformat.sign import NotebookNotary
 from traitlets import (
     Dict, Unicode, Integer, List, Bool, Bytes, Instance,
-    TraitError, Type,
+    TraitError, Type, Float
 )
 from ipython_genutils import py3compat
-from IPython.paths import get_ipython_dir
 from jupyter_core.paths import jupyter_runtime_dir, jupyter_path
 from notebook._sysinfo import get_sys_info
 
-from .utils import url_path_join, check_pid
+from .utils import url_path_join, check_pid, url_escape
 
 #-----------------------------------------------------------------------------
 # Module globals
 #-----------------------------------------------------------------------------
 
 _examples = """
-ipython notebook                       # start the notebook
-ipython notebook --profile=sympy       # use the sympy profile
-ipython notebook --certfile=mycert.pem # use SSL/TLS certificate
+jupyter notebook                       # start the notebook
+jupyter notebook --certfile=mycert.pem # use SSL/TLS certificate
 """
 
 #-----------------------------------------------------------------------------
@@ -118,15 +115,6 @@ def load_handlers(name):
     return mod.default_handlers
 
 
-class DeprecationHandler(IPythonHandler):
-    def get(self, url_path):
-        self.set_header("Content-Type", 'text/javascript')
-        self.finish("""
-            console.warn('`/static/widgets/js` is deprecated.  Use `/nbextensions/widgets/widgets/js` instead.');
-            define(['%s'], function(x) { return x; });
-        """ % url_path_join('nbextensions', 'widgets', 'widgets', url_path.rstrip('.js')))
-        self.log.warn('Deprecated widget Javascript path /static/widgets/js/*.js was used')
-
 #-----------------------------------------------------------------------------
 # The Tornado web application
 #-----------------------------------------------------------------------------
@@ -160,7 +148,9 @@ class NotebookWebApplication(web.Application):
             _template_path = (_template_path,)
         template_path = [os.path.expanduser(path) for path in _template_path]
 
-        jenv_opt = jinja_env_options if jinja_env_options else {}
+        jenv_opt = {"autoescape": True}
+        jenv_opt.update(jinja_env_options if jinja_env_options else {})
+
         env = Environment(loader=FileSystemLoader(template_path), **jenv_opt)
         
         sys_info = get_sys_info()
@@ -186,7 +176,13 @@ class NotebookWebApplication(web.Application):
                 'no_cache_paths': [url_path_join(base_url, 'static', 'custom')],
             },
             version_hash=version_hash,
-            
+            ignore_minified_js=ipython_app.ignore_minified_js,
+
+            # rate limits
+            iopub_msg_rate_limit=ipython_app.iopub_msg_rate_limit,
+            iopub_data_rate_limit=ipython_app.iopub_data_rate_limit,
+            rate_limit_window=ipython_app.rate_limit_window,
+
             # authentication
             cookie_secret=ipython_app.cookie_secret,
             login_url=url_path_join(base_url,'/login'),
@@ -221,7 +217,6 @@ class NotebookWebApplication(web.Application):
         
         # Order matters. The first handler to match the URL will handle the request.
         handlers = []
-        handlers.append((r'/deprecatedwidgets/(.*)', DeprecationHandler))
         handlers.extend(load_handlers('tree.handlers'))
         handlers.extend([(r"/login", settings['login_handler_class'])])
         handlers.extend([(r"/logout", settings['logout_handler_class'])])
@@ -240,16 +235,20 @@ class NotebookWebApplication(web.Application):
         handlers.extend(load_handlers('services.security.handlers'))
         
         # BEGIN HARDCODED WIDGETS HACK
+        # TODO: Remove on notebook 5.0
         try:
-            import ipywidgets
-            handlers.append(
-                (r"/nbextensions/widgets/(.*)", FileFindHandler, {
-                    'path': ipywidgets.find_static_assets(),
-                    'no_cache_paths': ['/'], # don't cache anything in nbextensions
-                }),
-            )
+            import widgetsnbextension
         except:
-            app_log.warn('ipywidgets package not installed.  Widgets are unavailable.')
+            try:
+                import ipywidgets as widgets
+                handlers.append(
+                    (r"/nbextensions/widgets/(.*)", FileFindHandler, {
+                        'path': widgets.find_static_assets(),
+                        'no_cache_paths': ['/'], # don't cache anything in nbextensions
+                    }),
+                )
+            except:
+                app_log.warning('Widgets are unavailable. Please install widgetsnbextension or ipywidgets 4.0')
         # END HARDCODED WIDGETS HACK
         
         handlers.append(
@@ -287,7 +286,7 @@ class NotebookWebApplication(web.Application):
 
 class NbserverListApp(JupyterApp):
     version = __version__
-    description="List currently running notebook servers in this profile."
+    description="List currently running notebook servers."
     
     flags = dict(
         json=({'NbserverListApp': {'json': True}},
@@ -324,7 +323,7 @@ flags['no-mathjax']=(
     {'NotebookApp' : {'enable_mathjax' : False}},
     """Disable MathJax
     
-    MathJax is the javascript library IPython uses to render math/LaTeX. It is
+    MathJax is the javascript library Jupyter uses to render math/LaTeX. It is
     very large, so you may want to disable it if you have a slow internet
     connection, or for offline use of the notebook.
     
@@ -346,6 +345,7 @@ aliases.update({
     'transport': 'KernelManager.transport',
     'keyfile': 'NotebookApp.keyfile',
     'certfile': 'NotebookApp.certfile',
+    'client-ca': 'NotebookApp.client_ca',
     'notebook-dir': 'NotebookApp.notebook_dir',
     'browser': 'NotebookApp.browser',
     'pylab': 'NotebookApp.pylab',
@@ -394,8 +394,10 @@ class NotebookApp(JupyterApp):
         """override default log format to include time"""
         return u"%(color)s[%(levelname)1.1s %(asctime)s.%(msecs).03d %(name)s]%(end_color)s %(message)s"
 
-    # create requested profiles by default, if they don't exist:
-    auto_create = Bool(True)
+    ignore_minified_js = Bool(False,
+            config=True,
+            help='Use minified JS file or not, mainly use during dev to avoid JS recompilation', 
+            )
 
     # file to be opened in the notebook server
     file_to_run = Unicode('', config=True)
@@ -468,6 +470,10 @@ class NotebookApp(JupyterApp):
         help="""The full path to a private key file for usage with SSL/TLS."""
     )
     
+    client_ca = Unicode(u'', config=True,
+        help="""The full path to a certificate authority certifificate for SSL/TLS client authentication."""
+    )
+    
     cookie_secret_file = Unicode(config=True,
         help="""The file where the cookie secret is stored."""
     )
@@ -541,8 +547,12 @@ class NotebookApp(JupyterApp):
     
     tornado_settings = Dict(config=True,
             help="Supply overrides for the tornado.web.Application that the "
-                 "IPython notebook uses.")
+                 "Jupyter notebook uses.")
     
+    cookie_options = Dict(config=True,
+        help="Extra keyword arguments to pass to `set_secure_cookie`."
+             " See tornado's set_secure_cookie docs for details."
+    )
     ssl_options = Dict(config=True,
             help="""Supply SSL options for the tornado HTTPServer.
             See the tornado docs for details.""")
@@ -558,7 +568,7 @@ class NotebookApp(JupyterApp):
     enable_mathjax = Bool(True, config=True,
         help="""Whether to enable MathJax for typesetting math/TeX
 
-        MathJax is the javascript library IPython uses to render math/LaTeX. It is
+        MathJax is the javascript library Jupyter uses to render math/LaTeX. It is
         very large, so you may want to disable it if you have a slow internet
         connection, or for offline use of the notebook.
 
@@ -606,8 +616,6 @@ class NotebookApp(JupyterApp):
         return [
             os.path.join(d, 'custom') for d in (
                 self.config_dir,
-                # FIXME: serve IPython profile while we don't have `jupyter migrate`
-                os.path.join(get_ipython_dir(), 'profile_default', 'static'),
                 DEFAULT_STATIC_FILES_PATH)
         ]
 
@@ -630,8 +638,13 @@ class NotebookApp(JupyterApp):
     def nbextensions_path(self):
         """The path to look for Javascript notebook extensions"""
         path = self.extra_nbextensions_path + jupyter_path('nbextensions')
-        # FIXME: remove IPython nbextensions path once migration is setup
-        path.append(os.path.join(get_ipython_dir(), 'nbextensions'))
+        # FIXME: remove IPython nbextensions path after a migration period
+        try:
+            from IPython.paths import get_ipython_dir
+        except ImportError:
+            pass
+        else:
+            path.append(os.path.join(get_ipython_dir(), 'nbextensions'))
         return path
 
     websocket_url = Unicode("", config=True,
@@ -647,9 +660,7 @@ class NotebookApp(JupyterApp):
     def _mathjax_url_default(self):
         if not self.enable_mathjax:
             return u''
-        static_url_prefix = self.tornado_settings.get("static_url_prefix",
-                         url_path_join(self.base_url, "static")
-        )
+        static_url_prefix = self.tornado_settings.get("static_url_prefix", "static")
         return url_path_join(static_url_prefix, 'components', 'MathJax', 'MathJax.js')
     
     def _mathjax_url_changed(self, name, old, new):
@@ -692,7 +703,7 @@ class NotebookApp(JupyterApp):
         of `jupyter_client.kernelspec.KernelSpecManager`.
 
         The Api of KernelSpecManager is provisional and might change
-        without warning between this version of IPython and the next stable one.
+        without warning between this version of Jupyter and the next stable one.
         """
     )
 
@@ -748,22 +759,40 @@ class NotebookApp(JupyterApp):
         else:
             return py3compat.getcwd()
 
+    def _notebook_dir_validate(self, value, trait):
+        # Strip any trailing slashes
+        # *except* if it's root
+        _, path = os.path.splitdrive(value)
+        if path == os.sep:
+            return value
+
+        value = value.rstrip(os.sep)
+
+        if not os.path.isabs(value):
+            # If we receive a non-absolute path, make it absolute.
+            value = os.path.abspath(value)
+        if not os.path.isdir(value):
+            raise TraitError("No such notebook dir: %r" % value)
+        return value
+
     def _notebook_dir_changed(self, name, old, new):
         """Do a bit of validation of the notebook dir."""
-        if not os.path.isabs(new):
-            # If we receive a non-absolute path, make it absolute.
-            self.notebook_dir = os.path.abspath(new)
-            return
-        if not os.path.isdir(new):
-            raise TraitError("No such notebook dir: %r" % new)
-        
         # setting App.notebook_dir implies setting notebook and kernel dirs as well
         self.config.FileContentsManager.root_dir = new
         self.config.MappingKernelManager.root_dir = new
 
+    # TODO: Remove me in notebook 5.0
     server_extensions = List(Unicode(), config=True,
-        help=("Python modules to load as notebook server extensions. "
-              "This is an experimental API, and may change in future releases.")
+        help=("DEPRECATED use the nbserver_extensions dict instead")
+    )
+    def _server_extensions_changed(self, name, old, new):
+        self.log.warning("server_extensions is deprecated, use nbserver_extensions")
+        self.server_extensions = new
+    
+    nbserver_extensions = Dict({}, config=True,
+        help=("Dict of Python modules to load as notebook server extensions."
+              "Entry values can be used to enable and disable the loading of"
+              "the extensions.")
     )
 
     reraise_server_extension_failures = Bool(
@@ -772,9 +801,20 @@ class NotebookApp(JupyterApp):
         help="Reraise exceptions encountered loading server extensions?",
     )
 
+    iopub_msg_rate_limit = Float(0, config=True, help="""(msg/sec)
+        Maximum rate at which messages can be sent on iopub before they are
+        limited.""")
+
+    iopub_data_rate_limit = Float(0, config=True, help="""(bytes/sec)
+        Maximum rate at which messages can be sent on iopub before they are
+        limited.""")
+
+    rate_limit_window = Float(1.0, config=True, help="""(sec) Time window used to 
+        check the message and data rate limits.""")
+
     def parse_command_line(self, argv=None):
         super(NotebookApp, self).parse_command_line(argv)
-        
+
         if self.extra_args:
             arg0 = self.extra_args[0]
             f = os.path.abspath(arg0)
@@ -784,7 +824,7 @@ class NotebookApp(JupyterApp):
                 self.exit(1)
             
             # Use config here, to ensure that it takes higher priority than
-            # anything that comes from the profile.
+            # anything that comes from the config dirs.
             c = Config()
             if os.path.isdir(f):
                 c.NotebookApp.notebook_dir = f
@@ -839,6 +879,7 @@ class NotebookApp(JupyterApp):
         if self.allow_origin_pat:
             self.tornado_settings['allow_origin_pat'] = re.compile(self.allow_origin_pat)
         self.tornado_settings['allow_credentials'] = self.allow_credentials
+        self.tornado_settings['cookie_options'] = self.cookie_options
         # ensure default_url starts with base_url
         if not self.default_url.startswith(self.base_url):
             self.default_url = url_path_join(self.base_url, self.default_url)
@@ -855,12 +896,19 @@ class NotebookApp(JupyterApp):
             ssl_options['certfile'] = self.certfile
         if self.keyfile:
             ssl_options['keyfile'] = self.keyfile
+        if self.client_ca:
+            ssl_options['ca_certs'] = self.client_ca
         if not ssl_options:
             # None indicates no SSL config
             ssl_options = None
         else:
-            # Disable SSLv3, since its use is discouraged.
-            ssl_options['ssl_version']=ssl.PROTOCOL_TLSv1
+            # SSL may be missing, so only import it if it's to be used
+            import ssl
+            # Disable SSLv3 by default, since its use is discouraged.
+            ssl_options.setdefault('ssl_version', ssl.PROTOCOL_TLSv1)
+            if ssl_options.get('ca_certs', False):
+                ssl_options.setdefault('cert_reqs', ssl.CERT_REQUIRED)
+        
         self.login_handler_class.validate_security(self, ssl_options=ssl_options)
         self.http_server = httpserver.HTTPServer(self.web_app, ssl_options=ssl_options,
                                                  xheaders=self.trust_xheaders)
@@ -871,7 +919,7 @@ class NotebookApp(JupyterApp):
                 self.http_server.listen(port, self.ip)
             except socket.error as e:
                 if e.errno == errno.EADDRINUSE:
-                    self.log.info('The port %i is already in use, trying another random port.' % port)
+                    self.log.info('The port %i is already in use, trying another port.' % port)
                     continue
                 elif e.errno in (errno.EACCES, getattr(errno, 'WSAEACCES', errno.EACCES)):
                     self.log.warn("Permission to listen on port %i denied" % port)
@@ -984,28 +1032,47 @@ class NotebookApp(JupyterApp):
         
         The extension API is experimental, and may change in future releases.
         """
+        
+        # TODO: Remove me in notebook 5.0
         for modulename in self.server_extensions:
-            try:
-                mod = importlib.import_module(modulename)
-                func = getattr(mod, 'load_jupyter_server_extension', None)
-                if func is not None:
-                    func(self)
-            except Exception:
-                if self.reraise_server_extension_failures:
-                    raise
-                self.log.warn("Error loading server extension %s", modulename,
-                              exc_info=True)
-    
+            # Don't override disable state of the extension if it already exist
+            # in the new traitlet
+            if not modulename in self.nbserver_extensions:
+                self.nbserver_extensions[modulename] = True
+        
+        for modulename in self.nbserver_extensions:
+            if self.nbserver_extensions[modulename]:
+                try:
+                    mod = importlib.import_module(modulename)
+                    func = getattr(mod, 'load_jupyter_server_extension', None)
+                    if func is not None:
+                        func(self)
+                except Exception:
+                    if self.reraise_server_extension_failures:
+                        raise
+                    self.log.warning("Error loading server extension %s", modulename,
+                                  exc_info=True)
+
+    def init_mime_overrides(self):
+        # On some Windows machines, an application has registered an incorrect
+        # mimetype for CSS in the registry. Tornado uses this when serving
+        # .css files, causing browsers to reject the stylesheet. We know the
+        # mimetype always needs to be text/css, so we override it here.
+        mimetypes.add_type('text/css', '.css')
+
     @catch_config_error
     def initialize(self, argv=None):
         super(NotebookApp, self).initialize(argv)
         self.init_logging()
+        if self._dispatching:
+            return
         self.init_configurables()
         self.init_components()
         self.init_webapp()
         self.init_terminals()
         self.init_signal()
         self.init_server_extensions()
+        self.init_mime_overrides()
 
     def cleanup_kernels(self):
         """Shutdown all kernels.
@@ -1020,7 +1087,7 @@ class NotebookApp(JupyterApp):
         "Return the current working directory and the server url information"
         info = self.contents_manager.info_string() + "\n"
         info += "%d active kernels \n" % len(self.kernel_manager._kernels)
-        return info + "The IPython Notebook is running at: %s" % self.display_url
+        return info + "The Jupyter Notebook is running at: %s" % self.display_url
 
     def server_info(self):
         """Return a JSONable dict of information about this server."""
@@ -1050,7 +1117,7 @@ class NotebookApp(JupyterApp):
                 raise
 
     def start(self):
-        """ Start the IPython Notebook server app, after initialization
+        """ Start the Notebook server app, after initialization
         
         This method takes no arguments so all configuration and initialization
         must be done prior to calling this method."""
@@ -1076,9 +1143,10 @@ class NotebookApp(JupyterApp):
                     self.exit(1)
 
                 relpath = os.path.relpath(self.file_to_run, self.notebook_dir)
-                uri = url_path_join('notebooks', *relpath.split(os.sep))
+                uri = url_escape(url_path_join('notebooks', *relpath.split(os.sep)))
             else:
-                uri = 'tree'
+                # default_url contains base_url, but so does connection_url
+                uri = self.default_url[len(self.base_url):]
             if browser:
                 b = lambda : browser.open(url_path_join(self.connection_url, uri),
                                           new=2)
@@ -1108,8 +1176,8 @@ class NotebookApp(JupyterApp):
 def list_running_servers(runtime_dir=None):
     """Iterate over the server info files of running notebook servers.
     
-    Given a profile name, find nbserver-* files in the security directory of
-    that profile, and yield dicts of their information, each one pertaining to
+    Given a runtime directory, find nbserver-* files in the security directory,
+    and yield dicts of their information, each one pertaining to
     a currently running notebook server instance.
     """
     if runtime_dir is None:
